#include <xc.h>     // XC8 compilateur
#include <stdint.h>
//#include <pic16f1847.h> // uint8_t dfinitions

#include "system.h"

// Fonction d'initialisation du microcontroleur
void Init_System(void)
{
// Oscillateur configur en 32MHz)
    OSCCON = 0x70;
    
// Rsistances de Pull UP actives
    OPTION_REGbits.nWPUEN = 0;
    WPUBbits.WPUB0 = 1;       // Pull Up sur RB0
    WPUBbits.WPUB3 = 1;       // Pull Up sur RB3
    
// Toutes les lignes analogiques sont bloques
    ANSELA = 0x00;
    ANSELB = 0x00;
    
// Toutes les registres des ports A et B sont initialises  0;
    LATA = 0x00;
    LATB = 0x00;
    
// Toutes les lignes des ports A et B sont configures en entre
    TRISA = 0xFF;
    TRISB = 0xFF;
    
// TODO 2 -> Dfinir le sens des lignes utilises
    DIR_DEL = 0;
    
//    DIR_MG = 0;
//    DIR_MLI_MG = 0;
//    DIR_MD = 0;
//    DIR_MLI_MD = 0;
    
    DIR_BP = 1;
    DIR_A2 = 1;
    DIR_A7 = 1;
    DIR_B0 = 1;
    DIR_B3 = 1;
    
// TODO 3 -> Choisir le niveau par dfaut des lignes en sortie
    DEL = 0;
    MG = 0;
    MLI_MG = 0;
    MD = 0;
    MLI_MD = 0;
    
//    for (uint8_t i=0; i< 10; i++)
//    {
//        DEL_CLI;
//        delayMs(100);
//    }
//    DEL_OFF;
}

// Fonction de temporisation en msec.
void delayMs(uint16_t ms)
{
    while((ms--))
    {
        __delay_ms(1);
    }
}

/*  Initialisation du Convertisseur Analogique Digital ADC */
void Init_ADC (void)
{
    ANSELA = 0b00000001;        // AN0 en mode Analogique
    ADCON1bits.ADNREF = 0;      // Ref Vdd-Vss
    ADCON1bits.ADPREF = 0;
    ADCON1bits.ADFM = 0;        // Justifi a gauche (mode 8bis)
    ADCON1bits.ADCS = 0b111;    // TAD 16 TOSC (2s @32MHz)
    ADCON0bits.CHS = 0;         // Slectionne le canal 0 -> AN0
    ADCON0bits.ADON = 1;        // ADC Module enable
}

/*  Fonction qui permet de choisir la ligne pour l' ADC */
//void SelChanADC (uint8_t canal)
//{
//    ADCON0bits.CHS = canal;
//}

/*  Lecture de l'entre analogique slectionne */
uint8_t ReadAdc8Bits (void)
{
    ADCON0bits.GO_nDONE = 1;
    while (ADCON0bits.GO_nDONE);
    return ADRESH;
}

/*  Paramtrage du Timer0 */
void Init_Timer0(void)
{
    OPTION_REGbits.TMR0CS = 0;      // Compteut sur Fosc/4
    OPTION_REGbits.PSA = 0;         // Prescaler vers Timer0
    OPTION_REGbits.PS = TMR0_PRE;   // Choix du Prescaler
    TMR0 = TMR0_VALUE;              // Prchargement du Timer0
    INTCONbits.TMR0IF = 0;          // Effacement du "flag" du Timer0
}

/*  Fonction de synchronisation de la boucle infinie dans le fichier main.c */
void Synchronisation(void)
{
    while (!INTCONbits.TMR0IF);     // Attente dbordement du Timer0
    TMR0 = TMR0_VALUE;              // Prchargement du Timer0
    INTCONbits.TMR0IF = 0;          // Effacement du "flag" du Timer0
}

/*  Initialisation de l'UART pour le dialogue C <-> PC */
void Init_UART(void)
{
    DIR_TX = 0;           // ligne TX en sortie
    DIR_RX = 1;           // Ligne RX en entre
    APFCON0bits.RXDTSEL = 1;        // RX to RB2
    APFCON1bits.TXCKSEL = 1;        // TX to RB5

    BAUDCONbits.BRG16 = 1;          // BRG16 = 1
    RCSTA = 0x90;                   // CREN = 1, SPEN = 1
    TXSTA = 0x24;                   // SYNC = 0; BRGH = 1
    SPBRGL = 138;                   // 57600
    SPBRGH = 0;
}

/*  Pour crire un seul caractre */
void putch(char c)
{
    while(PIR1bits.TXIF == 0);
    TXREG = c;
}

/*  Pour lire un seul caractre */
//char getch(void)
//{
//    while (!PIR1bits.RCIF);
//    return RCREG;
//}

void Init_Timer2(void)
{
//    CCPTMRS0bits.C3TSEL = 0;        // Timer2 pour PWM
    T2CONbits.T2CKPS = 0b10;        // Prescaler 1/4
    PR2 = 199;                      // Periode : (PR2 + 1)*prescaler*125ns -> 2,5kHz
    T2CONbits.TMR2ON = 1;           // Timer2 activ
}

/*  Intialisation des registres de contrles des moteurs */
void Init_MLI(void)
{
//    CCP1CON = 0x1C;         // Mode PWM
//    CCPR1 = 0;
//    CCP2CON = 0x1C;         // Mode PWM
//    CCPR2 = 0;
//    CCP3CON = 0x1C;         // Mode PWM
//    CCPR3 = 0;
//    CCP4CON = 0x1C;         // Mode PWM
//    CCPR4 = 0;
//    Init_Timer2();
}
